@@ -15,16 +15,16 @@ import shutil
 import subprocess
 import tarfile
 import tempfile
-import urllib2
 import urllib
+import requests
 from StringIO import StringIO
-from base64 import b64encode
 
 from getpass import getpass
 from subprocess import PIPE, STDOUT
 
 from sprinter.exceptions import (CommandMissingException,
                                  BadCredentialsException,
+                                 CertificateException,
                                  ExtractException,
                                  SprinterException)
 
@@ -58,19 +58,28 @@ def get_subclass_from_module(module, parent_class):
 
 
 def call(command, stdin=None, stdout=PIPE, env=os.environ, cwd=None, shell=False,
-         output_log_level=logging.INFO, logger=LOGGER):
+         output_log_level=logging.INFO, logger=LOGGER, sensitive_info=False):
     """ Better, smarter call logic """
-    args = command if shell else whitespace_smart_split(command)
-    kw = {}
-    if not shell and not which(args[0], cwd=cwd):
-        raise CommandMissingException(args[0])
-    if shell:
-        kw['shell'] = True
-    process = subprocess.Popen(args, stdin=PIPE, stdout=stdout, stderr=STDOUT,
-                               env=env, cwd=cwd, **kw)
-    output = process.communicate(input=stdin)[0]
-    logger.log(output_log_level, output)
-    return (process.returncode, output)
+    logger.debug("calling command: %s" % command)
+    try:
+        args = command if shell else whitespace_smart_split(command)
+        kw = {}
+        if not shell and not which(args[0], cwd=cwd):
+            raise CommandMissingException(args[0])
+        if shell:
+            kw['shell'] = True
+        process = subprocess.Popen(args, stdin=PIPE, stdout=stdout, stderr=STDOUT,
+                                   env=env, cwd=cwd, **kw)
+        output = process.communicate(input=stdin)[0]
+        logger.log(output_log_level, output)
+        return (process.returncode, output)
+    except OSError, e:
+        if not sensitive_info:
+            logger.exception("Error running command: %s" % command)
+            logger.error("Root directory: %s" % cwd)
+            if stdin:
+                logger.error("stdin: %s" % stdin)
+        raise e
 
 
 def __process(arg):
@@ -139,21 +148,19 @@ def whitespace_smart_split(command):
     return return_array
 
 
-def authenticated_get(username, password, url):
+def authenticated_get(username, password, url, verify=True):
     """
     Perform an authorized query to the url, and return the result
     """
     try:
-        request = urllib2.Request(url)
-        base64string = b64encode((b"%s:%s" % (username, password)).decode("ascii"))
-        request.add_header("Authorization", "Basic %s" % base64string)
-        result = urllib2.urlopen(request)
-    except urllib2.HTTPError, e:
-        if e.code == 401:
+        response = requests.get(url, auth=(username, password), verify=verify)
+        if response.status_code == 401:
             raise BadCredentialsException(
                 "Unable to authenticate user %s to %s with password provided!"
                 % (username, url))
-    return result.read()
+    except requests.exceptions.SSLError:
+        raise CertificateException("Unable to verify certificate at %s!" % url)
+    return response.content
 
 
 def prompt(prompt_string, default=None, secret=False, boolean=False):
@@ -233,7 +240,9 @@ def which(program, cwd=None):
 def extract_targz(url, target_dir, remove_common_prefix=False, overwrite=False):
     """ extract a targz and install to the target directory """
     try:
-        gz = gzip.GzipFile(fileobj=StringIO(urllib.urlopen(url).read()))
+        if not os.path.exists(target_dir):
+            os.makedirs(target_dir)
+        gz = gzip.GzipFile(fileobj=StringIO(requests.get(url).content))
         tf = tarfile.TarFile(fileobj=gz)
         if not os.path.exists(target_dir):
             os.makedirs(target_dir)
@@ -251,14 +260,16 @@ def extract_targz(url, target_dir, remove_common_prefix=False, overwrite=False):
                     else:
                         return
                 tf.extract(tfile, target_dir)
-    except OSError:
-        raise ExtractException()
-    except IOError:
-        raise ExtractException()
+    except OSError, e:
+        raise ExtractException(str(e))
+    except IOError, e:
+        raise ExtractException(str(e))
 
 
 def extract_zip(url, target_dir, remove_common_prefix=False, overwrite=False):
     try:
+        if not os.path.exists(target_dir):
+            os.makedirs(target_dir)
         memory_file = io.BytesIO(urllib.urlopen(url).read())
         zip_file = zipfile.ZipFile(memory_file)
         common_prefix = os.path.commonprefix(zip_file.namelist())
@@ -282,9 +293,11 @@ def extract_zip(url, target_dir, remove_common_prefix=False, overwrite=False):
 
 def extract_dmg(url, target_dir, remove_common_prefix=False, overwrite=False):
     if remove_common_prefix:
-        raise("Remove common prefix for zip not implemented yet!")
+        raise Exception("Remove common prefix for zip not implemented yet!")
     tmpdir = tempfile.mkdtemp()
     try:
+        if not os.path.exists(target_dir):
+            os.makedirs(target_dir)
         temp_file = os.path.join(tmpdir, "temp.dmg")
         urllib.urlretrieve(url, temp_file)
         call("hdiutil attach %s -mountpoint /Volumes/a/" % temp_file)
@@ -318,6 +331,10 @@ def remove_path(target_path):
         os.unlink(target_path)
 
 
+def insert_environment_osx(**environment_variables):
+    """ Inject the environment variables desired into the osx environment """
+
+
 def _determine_overwrite(prompt, overwrite, path):
     """ Determines if an overwrite is desired """
     if overwrite:
@@ -325,3 +342,16 @@ def _determine_overwrite(prompt, overwrite, path):
     elif prompt:
         return prompt("Path %s already exist! Overwrite?" % path, boolean=True)
     return True
+
+
+"""
+def get_file_owner_group()
+    stat_info = os.stat('/path')
+    uid = stat_info.st_uid
+    gid = stat_info.st_gid
+    print uid, gid
+
+    user = pwd.getpwuid(uid)[0]
+    group = grp.getgrgid(gid)[0]
+    print user, group
+"""
