@@ -10,8 +10,7 @@ The manifest can take a source and/or a target manifest
 """
 
 from ConfigParser import RawConfigParser
-import copy
-import logging
+import os
 import re
 import urllib
 from StringIO import StringIO
@@ -26,6 +25,7 @@ CONFIG_RESERVED = ['source', 'inputs']
 FEATURE_RESERVED = ['rc', 'command', 'phase']
 NAMESPACE_REGEX = re.compile('([a-zA-Z0-9_]+)(\.[a-zA-Z0-9_]+)?$')
 
+
 class ManifestException(Exception):
     pass
 
@@ -47,11 +47,13 @@ class Manifest(object):
     temporary_config_variables = []  # a list of the temporary keys, such as password
 
     def __init__(self, raw_manifest, namespace=None,
-                 logger=LOGGER, username=None, password=None):
+                 logger=LOGGER, username=None, password=None,
+                 verify_certificate=True):
         self.logger = logger
         self.manifest = self.__load_manifest(raw_manifest,
                                              username=username,
-                                             password=password)
+                                             password=password,
+                                             verify_certificate=verify_certificate)
         self.namespace = namespace or self.__parse_namespace()
         self.dtree = self.__generate_dependency_tree()
         self.system = System(logger=self.logger)
@@ -135,19 +137,24 @@ class Manifest(object):
         """ Add additional context variable """
         self.additional_context_variables = dict(self.additional_context_variables.items() + additonal_context.items())
 
-    def __load_manifest(self, raw_manifest, username=None, password=None):
+    def __load_manifest(self, raw_manifest, username=None, password=None, verify_certificate=True):
         manifest = RawConfigParser()
         manifest.add_section('config')
         if type(raw_manifest) == str:
             if raw_manifest.startswith("http"):
+                # raw_manifest is a url
                 if username and password:
                     manifest_file_handler = StringIO(lib.authenticated_get(username,
                                                                            password,
-                                                                           raw_manifest))
+                                                                           raw_manifest,
+                                                                           verify=verify_certificate))
                 else:
                     manifest_file_handler = StringIO(urllib.urlopen(raw_manifest).read())
                 manifest.readfp(manifest_file_handler)
             else:
+                # raw_manifest is a filepath
+                if not os.path.exists(os.path.expanduser(raw_manifest)):
+                    raise ManifestException("Manifest does not exist at %s!" % raw_manifest)
                 manifest.read(raw_manifest)
             if not manifest.has_option('config', 'source'):
                 manifest.set('config', 'source', str(raw_manifest))
